var Est94000_AprovacaoInventarioCego_Controller = function(templateManager, ScreenService, MessageService, General, Colors, EstUtil, ZHPromise, ZhFilterPreferences, localStorageEngine, Est94000_AprovacaoInventarioCego_Services) {

    let IDUTILALMOX, IDUTILLCESTQ, IDUTILLOTE;

    // Open initial screen filter with default values
    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est94000_AprovacaoInventarioCego_Services.getFilialLogada().then(success => {
            const dtInicial = moment().startOf('month').format('DD/MM/YYYY');
            const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

            rowsToSet.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(success => {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').getField("NMFILIAL").reload();
                this.verAlmoxEstoque(templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions'));
            });
        });
    };

    // Creates parameter bag to send to backend
    this.setFilterParams = function(widget, preference = null) {
        const row = preference.FILTER_ROW;

        let startDate = row.DTCONTAGEM.start;
        let endDate = row.DTCONTAGEM.end;
        let CDFILIAL = row.CDFILIAL;
        let CDALMOXARIFE = row.CDALMOXARIFE ? row.CDALMOXARIFE : ' ';
        let CDLOCALESTOQ = row.CDLOCALESTOQ ? row.CDLOCALESTOQ : ' ';

        let params = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": CDFILIAL
            },
            {
                "name": "IDSITESTCEGO",
                "operator": "=",
                "value": "DG"
            },
            {
                "name": "CDALMOXARIFE",
                "operator": "=",
                "value": CDALMOXARIFE
            },
            {
                "name": "CDLOCALESTOQ",
                "operator": "=",
                "value": CDLOCALESTOQ
            },
            {
                "name": "DTCONTAGEM",
                "operator": "BETWEEN",
                "value": [startDate, endDate]
            }
        ];

        if (CDALMOXARIFE == ' ')
            params = params.filter(field => {
                return field.name != 'CDALMOXARIFE' && field.name != 'CDLOCALESTOQ';
            });

        widget.dataSourceFilter = params;
        widget.reload();
    };

    // Almox and/or locale controller
    this.verAlmoxEstoque = function(args) {

        let widget = templateManager.container.getActiveWidget();

        if (widget.name != 'zhFilterPreferencesSelectConditions') widget = args;

        Est94000_AprovacaoInventarioCego_Services.getParamFilialData().then((filialParams) => {
            if (filialParams.dataset.paramfilial.length != 0) {

                // Set Global variables
                IDUTILALMOX = filialParams.dataset.paramfilial[0].IDUTILALMOX;
                IDUTILLCESTQ = filialParams.dataset.paramfilial[0].IDUTILLCESTQ;
                IDUTILLOTE = filialParams.dataset.paramfilial[0].IDUTILLOTE;

                let almoxField = widget.getField('DSALMOXARIFE');
                let LocalField = widget.getField('DSLOCALESTOQ');
                let cdAlmoxField = widget.getField('CDALMOXARIFE');
                let cdLocalEstField = widget.getField('CDLOCALESTOQ');

                widget.currentRow.CDALMOXARIFE = cdAlmoxField.value() ? cdAlmoxField.value() : "";

                //Verifies  warehouse control
                if (IDUTILALMOX === 'N') {
                    almoxField.readOnly = true;
                    almoxField.clearValue();
                    cdAlmoxField.clearValue("");
                    General.setInvisibleField('CDALMOXARIFE', 'inventoryGrid');

                }
                else {
                    General.setVisibleField('CDALMOXARIFE', 'inventoryGrid');
                    almoxField.readOnly = false;
                }

                //Verifies stock locale
                if (IDUTILLCESTQ === 'N') {
                    LocalField.readOnly = true;
                    LocalField.clearValue();
                    cdLocalEstField.clearValue("");
                    General.setInvisibleField('CDLOCALESTOQ', 'inventoryGrid');
                }
                else {
                    General.setVisibleField('CDLOCALESTOQ', 'inventoryGrid');
                    cdLocalEstField.clearValue();
                    LocalField.readOnly = false;
                }

                this.checkDataFilter(widget);
            }
        });
    };

    // Clear filter fields if unit field has change
    this.clearFieldsOnChangeFilial = function(widget) {
        let fields = [
            "DSALMOXARIFE",
            "DSLOCALESTOQ",
            "CDALMOXARIFE",
            "CDLOCALESTOQ"
            ];

        fields.forEach((field) => {
            let fieldToClear = widget.getField(field);
            fieldToClear.clearValue("");

            if (fieldToClear.name === "DSALMOXARIFE" || fieldToClear.name === "DSLOCALESTOQ") {
                fieldToClear.reload();
            }
        });
    };

    //This opens swipe details without change widget mode to view
    this.onOpenDetail = function(widget) {
        let ProductsGrid = templateManager.container.getWidget('productsGrid');

        ScreenService.openSwipe(ProductsGrid).then(() => {
            if (IDUTILLOTE == 'S') {
                General.setVisibleField('NRLOTEESTQ', ProductsGrid.name);
                General.setVisibleField('NRSUBLOTE', ProductsGrid.name);
            }
            else {
                General.setInvisibleField('NRLOTEESTQ', ProductsGrid.name);
                General.setInvisibleField('NRSUBLOTE', ProductsGrid.name);
            }
            ProductsGrid.reload();
        });
    };

    // This function saves approved inventories or send them back to retyping  situation 
    this.onChangeSituation = function(widget, situationAction) {

        const CheckedRows = widget.getCheckedRows();
        let message = situationAction == 'approved' ? "APPROVE_CONFIRMATION" : "RETYPE_CONFIRMATION";

        MessageService.confirmMessageByCode(message).then(_ => {
            Est94000_AprovacaoInventarioCego_Services.onChangeSituation(CheckedRows, situationAction).then((saved) => {
                widget.activate();
                widget.reload();
                if (situationAction == 'approved') MessageService.successNotificationByCode('APPROVE_SUCCESS');
            }).catch(error => {
                if (error.data) ScreenService.showMessage(error.data.error);
                else ScreenService.showMessage(error);
            });
        });
    };

    //It hides or show actions buttons to change the situation of inventories
    this.onValidateCheckedRowsHandler = function(widget) {
        const CheckedRows = widget.getCheckedRows();

        if (CheckedRows.length === 0) {
            General.setDisableAction('approved', widget.name);
            General.setDisableAction('retype', widget.name);
        }
        else {
            General.setEnableAction('approved', widget.name);
            General.setEnableAction('retype', widget.name);
        }
    };

    // It controls fields mode on filter
    this.checkDataFilter = function(args) {

        let widget = templateManager.container.getActiveWidget();

        if (widget.name != 'zhFilterPreferencesSelectConditions') {
            widget = args;
        }

        let LocalField = widget.getField('DSLOCALESTOQ');
        let cdLocalEstField = widget.getField('CDLOCALESTOQ');

        if (IDUTILALMOX === 'S' && IDUTILLCESTQ === 'S') {
            if (widget.currentRow.CDALMOXARIFE !== "") {
                LocalField.validations = { "required": { "message": "Campo Obrigatório" } };
                LocalField.readOnly = false;
            }
            else {
                LocalField.clearValue();
                cdLocalEstField.clearValue();
                delete LocalField.validations;
                LocalField.readOnly = true;
            }

            LocalField.reload();
        }

        if (IDUTILALMOX === 'S' && IDUTILLCESTQ === 'N') {
            if (widget.currentRow.CDALMOXARIFE !== "")
                cdLocalEstField.value(' ');
            else
                cdLocalEstField.clearValue("");
        }

    };

    // It creates icon in a row if quatity == zero
    this.IconHandler = function(widget) {
        let data = widget.dataSource.data;
        return data.map(row => {
            row.icon = row.QTINVENTESTCEGO == 0 ? "⚠" : '';
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est94000_AprovacaoInventarioCego_Controller', Est94000_AprovacaoInventarioCego_Controller);
});
